"""
anonipy
=========

Provides
  1. Label extractors
  2. Label generators
  3. Anonymization strategies

How to use the documentation
----------------------------
Documentation is available in two forms: docstrings provided
with the code and a loose standing reference guide, available
from `the anonipy homepage <https://eriknovak.github.io/anonipy>`.

"""

__version__ = "0.0.6"

from . import anonymize
from . import utils
from . import definitions
from . import constants
